
= Amrita2 quick start guide

== Step 1: hello world

This is the 'hello world' for Amrita2.

=== sources

--- rubysrc ../sample/hello/step1.rb

--- xmlsrc ../sample/hello/template.html

=== output 

--- xmlinline 
      # 
        <html>
          <body>
            <h1>hello world</h1>
            <p>Amrita2 is a html template library for Ruby</p>
          </body>
        </html>
  
=== description

There are two kind of objects in this example: Presentation Object(PO) and
Template Object(TO).

Amrita2::TemplateFile is a TO which accepts HTML template from a file.
This is how to create a Amrita2::TemplateFile object.

--- rubyinline 
      # 
        require "amrita2/template"
        include Amrita2
        tmpl = TemplateFile.new("template.html")

TO treats an element with id attribute as a dynamic element and
will get the data for it from PO data using id attribute's value
as method name.

Almost any Ruby object can be a Presentation Object only if PO has
methods for dynamic elements.


--- rubyinline 
      # 
         class PO
           def title
             "hello world"
           end
              
           def body
             "Amrita2 is a html template libraly for Ruby"
           end
         end

And a Hash can be used for PO.

--- rubyinline 
      # 
         data = {
           :title => "hello world",
           :body => "Amrita2 is a html template libraly for Ruby"
         }
         tmpl = TemplateFile.new("template.html")
         tmpl.expand(STDOUT, data)

If this data was given to the TO, same result will be produced.

== Step 2: Iteration and nested struct

PO methods returns a value for template expansion.
This sample describes how TO expand values.

=== sources

--- rubysrc ../sample/hello/step2.rb

=== output 

--- xmlinline 
      # 
        <html>
           <body>
              <h1>array data</h1>
              <span>1</span><span>2</span><span>3</span>
              <h1>nested struct</h1>
              <p>
                 <span>4</span>:<span>5</span>:<span>6</span>
              </p>
              <h1>array of nested struct</h1>
              <p>
                 <span>23</span>
              </p><p>
                 <span>24</span>
              </p>
           </body>
        </html>

=== Array expansion

If a passive PO method returns Array ( or an Enumerable object)

--- rubyinline 
      # 
        def array_data
          [1, 2, 3]
        end

then the matching element will be expanded many times.

From

--- xmlinline 
      # 
        <h1>array data</h1>
        <span id='array_data'> </span>

To

--- xmlinline 
      # 
        <h1>array data</h1>
        <span>1</span><span>2</span><span>3</span>

You can delete <span> tags ((<using Template Spec|URL:Spec.html>)) if you like.

=== Nested Struct

If the element has inner struct like this

--- xmlinline 
      # 
        <p id='nested_struct'>
           <span id='hour' />:<span id='min' />:<span id='sec' />
        </p>

then the PO method must return an object with the same methods with ID

--- rubyinline 
      # 
        def nested_struct
          Time.gm(2005,2,23, 4,5,6)
        end

A Time Object has hour, min, sec as method. These methods will be
called by TO and the result of the method will be inserted into
the matching element.

--- xmlinline 
      # 
        <h1>nested struct</h1>
        <p>
           <span>4</span>:<span>5</span>:<span>6</span>
        </p>

=== Array of Struct

If a PO method returns an Array of Struct

--- rubyinline 
     # 
      def nested_structs
        [Time.gm(2005,2,23, 4,5,6),Time.gm(2005,2,24, 4,5,6) ]
      end

then the element will be expanded two time each for an element of the Array.

From

--- xmlinline 
      # 
        <p id='nested_structs'>
           <span id='day' />
        </p>

To

--- xmlinline 
      # 
        <h1>array of nested struct</h1>
        <p>
           <span>23</span>
        </p><p>
           <span>24</span>
        </p>

This is true for any levels of nest and any combination of struct and array.

== Step 3: attribute handling
=== sources

Amrita2 has many ways to attribute handling.

--- rubysrc ../sample/hello/step3.rb

=== output 

--- xmlinline 
      # 
        <html>
           <body>
             <h1><a href='http://amrita2.rubyforge.org/'>replaced</a></h1>
             <a href='http://amrita2.rubyforge.org/'>Amrita2</a>
             <p>See <a href='http://www.ruby-lang.org/ja/'>Ruby homepage(Japanese)</a> for detail
             </p>
             <a href='http://blade.nagaokaut.ac.jp/cgi-bin/scat.rb/ruby/ruby-talk/132204'>
               [ruby-talk:132204]
             </a>
             <a href='http://raa.ruby-lang.org/list.rhtml?name=amrita'>
               [amrita: HTML template library]
             </a>
           </body>
        </html>

===  AttrArray object

AttrArray is a Object for a PO require TO to replace attributes.

If PO passes a AttrArray Object ( a(..) { .. } is shortcut for it),

--- rubyinline 
     # 
       data = { 
         :a1 => a(:href=>'http://amrita2.rubyforge.org/') { 'replaced' },
       }

The attribute and text is inserted to the matching template element.

So this template with data above

--- xmlinline 
      # 
        <h1><a id='a1'>replacing attributes</a></h1>

makes the result:

--- xmlinline 
      # 
         <h1><a href='http://amrita2.rubyforge.org/'>replaced</a></h1>

If the body part ( { ... } ) was ommited,


--- rubyinline 
      # 
        data = { 
           :a1 => a(:href=>'http://amrita2.rubyforge.org/')
        }

only attribute will be replaced

--- xmlinline 
      # 
        <a href='http://amrita2.rubyforge.org/'>replacing attributes</a>


=== amrita:type attribute

If you put a amrita:type attribute in the template,

--- xmlinline 
      # 
        <a id='a2' amrita:type='link'>Amrita2</a>

Amrita2 will treat the element in a diffrent way. In this case, the
type is 'link' which means the value will be inserted to the href part
of the element instead of the contained text of the element. 

So you can pass just the url alone for the element.

--- rubyinline 
     # 
       data = { 
         :a2 => 'http://amrita2.rubyforge.org/',
       }

The result is

--- xmlinline 
      # 
       <a href='http://amrita2.rubyforge.org/'>Amrita2</a>

=== use_args

amrita:type='use_args' means '$1' in
template will be replaced with the data given.

--- xmlinline 
      # 
         <a id='rubytalk' amrita:type='use_args' 
           href="http://blade.nagaokaut.ac.jp/cgi-bin/scat.rb/ruby/ruby-talk/$1">
           [ruby-talk:$1]
         </a>

If you give this data to this template above

--- rubyinline 
      # 
        data {
          :rubytalk=>132204,
        }

the result will be
      
--- xmlinline 
      # 
        <a href='http://blade.nagaokaut.ac.jp/cgi-bin/scat.rb/ruby/ruby-talk/132204'>
          [ruby-talk:132204]
        </a>

If two or more args are needed

--- xmlinline 
      # 
        <a id='raa'  amrita:type='use_args' 
           href="http://raa.ruby-lang.org/list.rhtml?name=$1">
           [$2]
        </a>

Give an Array for it.

--- rubyinline 
      # 
        data {
          :raa =>['amrita', 'amrita: HTML template library']
        }


The result is

--- xmlinline 
      # 
       <a href='http://raa.ruby-lang.org/list.rhtml?name=amrita'>
         [amrita: HTML template library]
       </a>

=== use_original_element

If you use an attribute or text in the template and make the result from it,
use the amrita:type 'use_original_element'.

--- xmlinline 
      # 
         <p>See <a id='a3' amrita:type='use_original_element' 
           href='http://www.ruby-lang.org/'>Ruby homepage</a> for detail
         </p>

And pass a Proc to it,

--- rubyinline 
      # 
          data = {
            :a3 => proc do |e|
              # A REXML::Element will be passed to the proc
              e.text += "(Japanese)"
              e.attributes['href'] += 'ja/'
              # modify it or make new Element and return it
              e
            end,
          }

The result is 

--- xmlinline 
      # 
       <p>See <a href='http://www.ruby-lang.org/ja/'>Ruby homepage(Japanese)</a> for detail
       </p>


=== 'amrita:type's

There are many 'amrita:type's

:link
      for <a href='....'>....</a>
:use_args
      replacing $1,$2,$3 with args given
:use_original_element
      modify the original element in a Proc
:input
      for <input .... />. The value is inserted as a 'value' attribute
:checkbox
      for <input type='checkbox' .... />. The value is inserted as "checed='checked'"
:select
      for <select ..><option ...><option ...></select>
:erb
      ERB processing in an Amrita2 template (desribed later)

And you can add new types to Amrita2 if you like to.

The interface of this plugin is very complicated and not fixed yet.
So it is not documented well.
Mail me if you want to use this feature.
I may write a plugin or base of it for you.

--- rubysrc ../sample/spec/inline.rb


== Step 4: ERB processing

=== sources

--- rubysrc ../sample/spec/erb.rb

=== How to use ERB in Amrita2

Use amrita:type='erb' and put the partial ERB source in
![CDATA[.....]]. Because <% %> is not allowed in XML.

--- xmlinline 
      # 
         <span id='header_title' amrita:type='erb'><![CDATA[
           <title><%= TITLE %></title>
         ]]></span>

And pass a Binding Object to it.

--- rubyinline 
      # 
        TITLE='Amrita2 with ERB '
        b = binding 
        data = { 
          :header_title => b,
        }

Get the Binding at the place where all variables used in the ERB
template are visible. 

In this case, a constant 'TITLE' is used by the ERB source, so binding
should be called where TITLE is visible.

=== pass a value to ERB

You can pass a value to ERB element.

--- rubyinline 
      # 
        data = { 
          :title => [b, ' Sample'], # pass an addtional args
        }

The string ' Sample' will be assigned to $_ and you can use it
in ERB source.

--- xmlinline 
      # 
          <span id='title' amrita:type='erb'><![CDATA[
            <h1><%= TITLE + $_ %></h1>
          ]]></span>

=== making a table with ERB

If you want to construct an element with a method like this,

--- rubyinline 
      # 
        def make_link(url, text)
          "<a href='#{url}'>#{text}</a>"
        end

get the binding where you can use the method, and pass the data for it

--- rubyinline 
      # 
        data {
          :item => [binding, ["Ruby", "http://www.ruby-lang.org/", "Matz"  ] ]
        }

This data and method can be used in the ERB source

--- xmlinline 
      # 
        <span id='item' amrita:type='erb'><![CDATA[
        <tr>
          <td><%= make_link($_[1], $_[0]) %></td>
          <td><%= $_[2] %></td>
        </tr>
        ]]></span>

The result is 

--- xmlinline 
      # 
        <tr>
          <td><a href='http://www.ruby-lang.org/'>Ruby</a></td>
          <td>Matz</td>
        </tr>

You can make a table if you pass a array of pair of binding and data.

--- rubyinline 
      # 
        table = [
          ["Ruby", "http://www.ruby-lang.org/", "Matz"  ],
          ["Perl", "http://www.perl.com/",  "Larry Wall" ],
          ["Python", "http://www.python.org/", "Guido van Rossum"   ]
        ]
        b = binding 
        data = { 
          :item=> table.collect { |i| [b, i] }
        }


== next step

=== for Ruby on Rails Programer

* ((<How to use Amrita2 with Ruby On Rails|URL:Rails.html>))

=== for programmers co-working with designers or Dreamweaver guys

* ((<using Template Spec|URL:Spec.html>))
* ((<Reverse Engineering|URL:Reverse.html>))

=== for who need good performance and like to know internals of Amrita2

* ((<active po methods|URL:active_po.html>))

=== for who loves Amrita2 and like to know every details of it

* ((<A2TDL|URL:A2TDL.html>))
* Parts template
* Dialog System
* Encoding 
* RDA: RD with Amrita2 ( This document was generated with this )
* ((<JavaScript Template|URL:js.html>))

