= How to use Amrita2 with Ruby On Rails

== Summary

Amrita2 provides a Rails-Amrita2 bridge based on the work of of Jamis Buck.

  * http://one.textdrive.com/pipermail/rails/2005-February/003169.html
  * http://ruby.jamisbuck.org/amrita-view.zip

There are two modules of template delegator.

:XHTML delegator
   delegator for XHTML template

:TDL delegator
   delegator for A2TDL template

XHTML delegator replaces ERB template system in most cases with a
little work.

A2TDL: Amrita2 Template Definition Language, is a builder type API of
Amrita2 . This is a experimental and new feature.

== Setup

First, install ROR and Amrita2.

Amrita2 has no installer now. So, it is the easiest way to copy
amrita2/lib to your Rails application's lib directory.

After making Rails application skelton with 'rails' command,
put the lines below to bottom of '(APPLICATION)/config/environtment.rb

      require 'amrita2/rails_bridge'
      ActionView::Base.register_template_handler "a2tdl", Amrita2TDLView::Base
      ActionView::Base.register_template_handler "a2html", Amrita2View::Base


If you put templates to (APPLICATION)/views/(CONTROLER)/ directory,
Amrita2-RAILS briged will render your template with data provided by controlers and helper methods.

== 'hello world' tutorial

Create a application skelton with this command.

  ./script/generate controler hello 

And setup the delegator adding the line below at the bottom of config/environtment.rb

      require 'amrita2/rails_bridge'
      ActionView::Base.register_template_handler "a2html", Amrita2View::Base

Start the application

  ruby script/server

=== Hello World: Views 

Create 'app/views/hello/index.a2html' and 

--- xmlinline 
      # 
        <html>
          <body>
            <h1 id='title'>hello world</h1>
            <p id='body'>Amrita2 is a html template library for Ruby</p>
          </body>
        </html>


You can access 'http://localhost:3000/' and see the template at this time.

=== Hello World: Controlers 

Edit 'controllers/hello_controller.rb' like this

--- rubyinline 
      # 
        class HelloController < ApplicationController
          def index
            @title = 'Hello ROR with Amrita2'
            @body = 'Using Amrita2 with ROR is easy and fun!'
          end
        end

Amrita2 checks instance variables of the same name of id attributes of
template.  So you only have to set instance variables of controlers to
show them with template.

Now, after editing, see the 'http://localhost:3000/' again.  You don't
have to restart the server.

If everything is OK, the result will be

--- xmlinline 
      # 
        <html>
          <body>
            <h1>Hello ROR with Amrita2</h1>
            <p>Using Amrita2 with ROR is easy and fun!</p>
          </body>
        </html>

As you can see, Amrita2 uses contorollers as model data and insert the
value of instance variables of it to matching element of the template.

=== using model and make tables

  ./script/generate model Language

app/models/language.rb

--- rubyinline 
      # 
        class Language < Struct.new(:name, :creator)
          def self.find_all
            [
              self.new("Ruby", "matz" ),
              self.new("perl", "Larry Wall" ),
              self.new("python", "Guido van Rossum")
            ]
          end
        end

controllers/hello_controller.rb

--- rubyinline 
      # 
        def languages
          @languages = Language.find_all
        end

views/hello/languages.a2html

--- xmlinline 
      # 
        <html>
        <body>
        <table border='1'>
          <tr><th>name</th><th>author</th></tr>
          <tr id='languages'>
            <td id='name'></td><td id='creator'></td>
          </tr>
        </table>
        </body>
        </html>

access 'http://localhost:3000/language', you will see this

--- xmlinline 
      # 
        <html>
        <body>
        <table border='1'>
          <tr><th>name</th><th>author</th></tr>
          <tr>
            <td>Ruby</td><td>matz</td>
          </tr><tr>
            <td>perl</td><td>Larry Wall</td>
          </tr><tr>
            <td>python</td><td>Guido van Rossum</td>
          </tr>
        </table>
        </body>
        </html>

Every indentation of output in this document was changed for
documenting.


== Using PO method

If some View Logic is needed, you can use PO method.

app/models/language.rb

--- rubyinline 
      # 
        class Language < Struct.new(:name, :creator, :url)
          def self.find_all
            [
              self.new("Ruby", "matz", "http://www.ruby-lang.org/"),
              self.new("perl", "Larry Wall", "http://www.perl.com/" ),
              self.new("python", "Guido van Rossum", "http://www.python.org/")
            ]
          end
        end

controllers/hello_controller.rb

--- rubyinline 
      # 
        def languages
          @languages = Language.find_all
        end


helpers/hello_helper.rb


--- rubyinline 
      # 
        module HelloHelper
          include Amrita2
          def languages_po
            { 
              :languages => controller.languages.collect do |l|
                { 
                  :name => a(:href=>l.url) { l.name },
                  :creator => l.creator
                }
              end
            }
          end
        end

If the view or helper classes have a method named 'XXXXXXX_po',
Amrita2-Rails bridge will call this method befor template expansion.
And use the result of it.

If the direct result of Models can not be used for template, 
you can modify or generate PO from the Model using PO method.


views/hello/languages.a2html

--- xmlinline 
      # 
        <html>
        <body>
        <table border='1'>
          <tr><th>name</th><th>author</th></tr>
          <tr id='languages'>
            <td><a id='name' /></td><td id='creator'></td>
          </tr>
        </table>
        </body>
        </html>


result

--- xmlinline 
      # 
        <html>
        <body>
        <table border='1'>
          <tr><th>name</th><th>author</th></tr>
          <tr>
            <td><a href='http://www.ruby-lang.org/'>Ruby</a></td>
            <td>matz</td>
          </tr><tr>
            <td><a href='http://www.perl.com/'>perl</a></td>
            <td>Larry Wall</td>
          </tr><tr>
            <td><a href='http://www.python.org/'>python</a></td>
            <td>Guido van Rossum</td>
          </tr>
        </table>
        </body>
        </html>

== using form_helper in Amrita2

Amrita2 can be used with ERB partialy. 
So you can use many useful helper methods of ActiveView.


To use if, set amrita:type='erb' in the template and
wrap the ERB template with <![CDATA[...]]>

--- xmlinline 
      # 
        <html>
        <body>
        <table border='1'>
          <tr><th>name</th><th>author</th></tr>
          <tr amrita:id='languages'>
            <td amrita:id='name'><a amrita:id='link' /></td><td amrita:id='creator'></td>
          </tr>
        </table>
        <hr />
        <div amrita:id='form' amrita:type='erb'><![CDATA[
        <%= form_tag({ :action => "insert" }) %>
        Name:
        <%= text_field "language", "name", "size" => 20 %><br/>
        Url:
        <%= text_field "language", "url", "size" => 20 %><br/>
        Creator:
        <%= text_field "language", "creator", "size" => 20 %><br/>
        
        <input type="submit" value="Add">
        ]]></div>
        
        </body>
        </html>

--- rubyinline 
      # 
        def languages_po
          b = binding
          { 
            :languages => controller.languages.collect do |l|
              { 
                :name => { :link => a(:href=>l.url) { l.name } },
                :creator => l.creator,
                :birthday => l.birthday
              }
            end,
            :form => b,
          }
        end



== using template setup method

If a method named '(method_name)_setup_template ' was defined in helper module,
Amrita2-ROR bridge will call this method for template compiling.

You can control detail of Amrita2 providing this method.

--- rubyinline 
      # 
        def languages_setup_template(template)
          tmpl = Amrita2::TemplateText.new(template)
          tmpl.amrita_id = 'amrita:id'
          tmpl
        end

With this template setup method, you can use 'amrita:id' attribute instead of 'id'.


== using A2TDL

A2TDL is a builder type API. It generates static XHTML template with
place holder for dynamic part.


hello/index.a2tdl

--- rubyinline 
      # 
        Amrita2::define_template_spec do 
          html do 
            title('Hello world')
            h2(:title)
            p(:body)
            h2('Static part')
            p('String is static part. Symbol is dynamic part')
          end
        end

With controllers/hello_controller.rb shown above, 
this template produce output like this.

--- xmlinline 
      # 
          <html>
          <head>
            <title>Hello world</title>
          </head>
          <body>
            <h1>Hello world</h1>
            <h2 id='title'>Hello ROR with Amrita2</h2>
            <p id='body'>Using Amrita2 with ROR is easy and fun!</p>
            <h2>Static part</h2>
            <p>String is static part. Symbol is dynamic part</p>
          </body>
          </html>

views/hello/languages.a2tdl
--- rubyinline 
      # 
        Amrita2::define_template_spec do 
          html do 
            title('Languages')
            simple_table(:languages, :table_opt => { :border=>1 }) do
              column(:name) do 
                a_link(:link)
              end
              column(:creator)
            end
          end
        end

helpers/hello_helper.rb

--- rubyinline 
      # 
        def languages_po
          { 
            :languages => controller.languages.collect do |l|
              { 
                :name => { :link => a(:href=>l.url) { l.name } },
                :creator => l.creator
              }
            end
          }
        end
      

--- xmlinline 
      # 
        <html>
        <head>
        <title>Languages</title>
        </head>
        <body><h1>Languages</h1><table border='1'>
        <tr><th>name</th><th>creator</th></tr>
        <tr>
          <td><span id='name'>
            <a href='http://www.ruby-lang.org/'>Ruby</a>
          </span></td>
          <td>matz</td>
        </tr>
        <tr>
          <td><span id='name'>
            <a href='http://www.perl.com/'>perl</a>
          </span></td>
          <td>Larry Wall</td>
        </tr>
        <tr>
          <td><span id='name'>
            <a href='http://www.python.org/'>python</a>
          </span></td>
          <td>Guido van Rossum</td>
        </tr>
        </table>
        </body>
        </html>


== using form_helper in Amrita2 and TDL

--- rubyinline 
      # 
        Amrita2::define_template_spec do 
          html do 
            title('Languages')
            simple_table(:languages, :table_opt => { :border=>1 }) do
              column(:name) do 
                a_link(:link)
              end
              column(:creator)
              column(:birthday)
            end
            
            grid_form(:form, :buttons=>[:add]) do
              erb(:name, '<%= text_field "language", "name", "size" => 20 %>')
              erb(:url, '<%= text_field "language", "url", "size" => 20 %>')
              erb(:creator, '<%= text_field "language", "creator", "size" => 20 %>')
              erb(:birthday, '<%=  date_select("language", "birthday",
                           , :order => [:year, :month, :day],:start_year => 1970) %>')
            end
          end
        end
  

--- rubyinline 
      # 
        def languages_tdl_po
          b = binding
          { 
            :languages => controller.languages.collect do |l|
              { 
                :name => { :link => a(:href=>l.url) { l.name } },
                :creator => l.creator,
                :birthday => l.birthday
              }
            end,
            :form => a(:action => url_for(:action => "insert_tdl")) do
               { 
                 :name => b,
                 :url => b,
                 :creator => b,
                 :birthday => b,
               }
            end
          }
        end


== calender component

template

--- rubyinline 
      # 
        Amrita2::define_template_spec do 
          html do
            title('calender')
            table do 
              tr(:navigation) do 
                td do 
                  a_link(:prev)
                end
                td(nil, :html_attr=>{ :align => 'right'}) do
                  a_link(:next)
                end 
              end
              tr do 
                td(nil, :html_attr=>{ :colspan => '2'}) do 
                  simple_calender(:cal) do 
                    cal_date do
                      span(:d)
                      div (:sched) do
                        a_link(:link)
                      end
                    end
                  end
                end
              end
            end
          end
        end
      
logic

--- rubyinline 
      # 
        class CalenderController < ApplicationController
          include Amrita2
          def index
            # get year and month
            .....

            set_navigation(y.to_i,m.to_i)
            set_calender(y.to_i,m.to_i)
          end
        
          def set_calender(y,m)
            @cal = TDL::Calender::Data.new(y,m) do |d|
              get_schedule(d)
            end
          end
        
          def set_navigation(y,m)
            # set navigation links
            .....
          end
        
          def get_schedule(d)
            sch = Schedule.find_by_date(d)
            if sch
              { 
                :sched => { 
                  :link => a(:href=>sch.url) {
                    sch.text 
                  },
                },
                :d => d.day
              }
            else
              { 
                :d => d.day
              }
            end
          end
        end

result
  

  <<<                         >>>
              2005/4
   Sun  Mon Tue  Wed  Thu Fri Sat
                            1   2
      3   4   5     6   7   8   9
     10  11  12    13  14  15  16
  test1
     17  18  19    20  21  22  23
                test2
     24  25  26    27  28  29  30
