= Using Template Spec 

== What is Template Spec ?

Amrita2 is an HTML-Ruby compiler and needs additional information to
compile a HTML template to Ruby modules. That is what we call Template
Spec .

== Why and When 

If Template Spec was not given, TO will generate a default Spec and
use it for compiling. If the default was not enough, you can give
a manually made Spec to TO.

* need to use a HTML template without id attributes
* need to validate HTML templates
* to use advanced features of Amrita2

And I'm planing to solve performance problems by using Specs later.


== How to use Template Spec

=== sample source

--- rubysrc ../sample/spec/spec.rb

=== description

--- rubyinline 
      # 
        tmpl = TemplateText.new(TEMPLATE_TEXT) do
          Amrita2::define_template_spec do 
            dynamic_element(:title)
            dynamic_element(:body, :matcher=>HtmlTag::P)
          end
        end

The block given to TemplateText.new is a Template Spec.

The top level must be 'Amrita2::define_template_spec' and consists of
some 'dynamic_element's .

'dynamic_element's must have a Symbol as id and optionaly a Hash as
options.

(({:matcher})) is an options for using an HTML element without id for
dynamic element. In this case, an element with TAG <p> will be used as
dynamic element named :body.

== amrita:type and Template Spec

amrita:type is a set of Spec options and supposed to be a shortcut for
Template Spec.

amrita:id's are defined in template.rb like this.

--- rubyinline 
      # 
          DefaultTypeMap = { 
            # amrita:id name => { Spec options ... }
            :use_args => { :use_args => true},
            :use_original_element=> { :use_original_element => true},
            :expand_by_member => { :expand_by_member => true },
            :value_attr => { :value_attr => true },
            :erb => { :erb => true },
            :link => { :link_attr => [:href, [/javascript:.*/]] },
            :input => { :value_attr => :value },
            :checkbox => { :value_attr => :checked },
            :select => { :selection_element => [:selected, []] },
          }

== Template Spec options

:key_attr_name
   attribute name for binding. Default is (({:id}))
:matcher
   matcher object for matching Template Element and dynamic element
:delete_tag
   If (({true})), the tag will be deleted in the output
:output_stream
   If specified, the output for this element will be inserted into the position of specified elements
:delete_id
   If (({true})), the attribute (({:key_attr_name})) will be deleted 
:place_holder
   (not documented)
:direct_define
   (not documented)
:match_many
   (not documented)
:generate_template
   (not documented)
:encoding
   (not documented)
:dummy_element
   (not documented)
:parts_element
   (not documented)
:ie_hack
   (not documented)
:check_true
   (not documented)
:use_original_element
   (not documented)
:expand_by_member
   (not documented)
:use_args
   (not documented)
:value_attr
   (not documented)
:link_attr
   (not documented)
:selection_element
   (not documented)
:erb
   (not documented)
:use_single_arg
   (not documented)
:use_erb_args
   (not documented)


== samples

=== Using Amrita2 with Template without id attributes

--- rubysrc ../sample/spec/matcher.rb

=== Using Expand by Member feature

--- rubysrc ../sample/spec/member.rb

=== validating template with Template Spec

=== debuging PO without template

=== Using Tables

=== Using <dl>


