require 'rubygems'
require 'amrita2/rd'
require 'syntax/convertors/html'

include Amrita2

class MyExtention < RDAStandardExtention
  TMPL = REXML::Document.new <<-'END'
  <div id='extention'>
    <a id='api' href="$1/$2">$1#$2</a>
    <div id='include_file'><table class='srcbox'>
      <tr><td align='center'><span class='srctitle'>$1</span></td></tr>
      <tr><td><pre>$0</pre></td></tr>
    </table></div>
    <div id='rubysrc'><table class='srcbox'>
      <tr><td align='center'><span class='srctitle'>$1</span></td></tr>
      <tr><td class='ruby'>$0</td></tr>
    </table></div>
    <div id='rubyinline'><table class='srcbox'>
      <tr><td class='ruby'>$0</td></tr>
    </table></div>
    <div id='xmlsrc'><table class='srcbox'>
      <tr><td align='center'><span class='srctitle'>$1</span></td></tr>
      <tr><td class='xml'>$0</td></tr>
    </table></div>
    <div id='xmlinline'><table class='srcbox'>
      <tr><td class='xml'>$0</td></tr>
    </table></div>
  </div>
  END

  SPEC = Amrita2::define_template_spec do 
    dynamic_element(:extention, :delete_id=>true) do
      parts_element(:api, :use_args=>true)
      parts_element(:include_file, :use_args=>true)
      parts_element(:rubysrc, :use_args=>true)
      parts_element(:rubyinline, :use_args=>true)
      parts_element(:xmlsrc, :use_args=>true)
      parts_element(:xmlinline, :use_args=>true)
    end
  end
  include SPEC.create_template_module(TMPL)

  def include_file(name, title=name)
    super("--- #{title} ---\n") { File::open(name).read }
  end

  RubyConvertor = Syntax::Convertors::HTML.for_syntax "ruby"
  def rubysrc(name, title=name)
    super("--- #{title} ---\n") do
      src = File::open(name).read 
      if /\A(.*)__END__(.*)\Z/m =~ src
        src, result = $1, $2
        Amrita2::SanitizedString[RubyConvertor.convert(src) +
            "output<span class='xml'>" + XMLConvertor.convert(result) + "</span>"]
      else
        Amrita2::SanitizedString[RubyConvertor.convert(src)]
      end
    end
  end

  def rubyinline(&block)
    super do
      src = unescape(yield)
      #p src
      Amrita2::SanitizedString[RubyConvertor.convert(src)]
    end
  end

  XMLConvertor = Syntax::Convertors::HTML.for_syntax "xml"
  def xmlsrc(name, title=name)
    super("--- #{title} ---\n") do
      src = File::open(name).read 
      Amrita2::SanitizedString[XMLConvertor.convert(src)]
    end
  end

  def xmlinline(&block)
    super do
      src = yield
      src = unescape(src)
      Amrita2::SanitizedString[XMLConvertor.convert(src)]
    end
  end

  def unescape(src)
    src = src.gsub(%r[<p>#\s*</p>], "")
    src.gsub!(%r[\A<pre>], "")
    src.gsub!(%r[</pre>\Z], "")
    src.gsub!("&lt;", "<")
    src.gsub!("&gt;", ">")
    src
  end
end

dir = ARGV.shift

Dir::chdir(dir) do
  Dir::glob("*.rd") do |f|
    #puts f
    ext = MyExtention.new
    to = File::basename(f, '.rd') + '.html'
    #puts to
    Amrita2::RDAProcessor.compile_rd(f, to, 
      :css=>"default.css", :extention_processor=>ext)
  end
end

