require 'erb'
require 'amrita2/template'
require 'amrita2/ext'

module Amrita2
  module TDL
    extend TextSupport

    module Meta
      include Core::SpecOptionMeta::MetaData
      def define_element_ext(spec, name, sym, params, &block)
        pp = params
        pp[name] = true
        pp[:html_attr] ||= {}
        spec.dynamic_element(sym, pp, HtmlElementSpec, &block)
      end
      Core::SpecOptionMeta::define_option(:html, self)
      Core::SpecOptionMeta::define_option(:div, self)
      Core::SpecOptionMeta::define_option(:title)
      Core::SpecOptionMeta::define_option(:css)
    end

    class HtmlElementSpec < Core::DynamicElementSpec
      def make_default_template
        return super unless @option[:html] 
        title = @option[:html_attr][:title]
        css = @option[:css]
        t = (@option[:meta_template] || TDL::meta_template(:html))
        t.expand(ret="") do |m|
          if title or css
            m.head do |mh|
              mh.title(title) if title
              mh.css(css) if css
            end
          end
          body = children.collect do |c|
            c.make_default_template
          end.join('')
          m.body(SanitizedString[body])
        end
        ret
      end

      def title(t)
        @option[:html_attr][:title] = t
        h1(t)
      end

      def simple_table(sym , params={}, &block)
        params[:table_opt] ||= {}
        params[:table_opt][:name] = sym 
        params[:delete_id] = true
        params[:match_many] = true if params[:table_opt][:classes] 
        dynamic_element(sym, params, TableElementSpec, &block)
      end

      def p(*args)
        define_html_element(:p, *args)
      end

      def p_(*args)
        Kernel::p(*args)
      end

      def a_link(name, param={}, &block)
        param[:link_attr] = [:href, [/javascript:.*/]]
        dynamic_element(name, param, LinkElementSpec, &block)
      end
      def input(name, typ, opt={})
        opt[:input_field] = true
        opt[:value_attr] = :value
        opt[:value_attr] = :checked if typ == :checkbox
        opt[:html_tag] = :input
        opt[:key_attr_name] = :name
        opt[:field_type] = typ
        dynamic_element(name, opt)
      end

      def text_field(name, opt={})
        input(name, :text, opt)
      end

      def hidden_field(name, opt={})
        input(name, :hidden, opt)
      end

      def password_field(name, opt={})
        input(name, :password, opt)
      end

      def submit(name, opt={})
        input(name, :submit, opt)
      end

      def check_box(name, opt={})
        opt[:value_attr] = :checked
        input(name, :checkbox, opt)
      end

      def select(name, opt={}, &block)
        opt[:html_tag] = :select
        opt[:key_attr_name] = :name
        opt[:selection_element] = [:selected, []]
        dynamic_element(name,opt, SelectionElementSpec, &block)
      end

      def radio_buttons(name, opt={}, &block)
        opt[:html_tag] = :fieldset
        opt[:key_attr_name] = :name
        opt[:selection_element] = [:checked, []]
        dynamic_element(name,opt, RadioButtonsElementSpec, &block)
      end

      def textarea(name, opt={})
        opt[:html_tag] = :textarea
        opt[:key_attr_name] = :name
        dynamic_element(name, opt)
      end

      def form_element(name, param={}, &block)
        param[:form_element] = true
        param[:html_tag] = :form
        dynamic_element(name, param, HtmlElementSpec, &block)
      end

      def erb(name, src, params={}, &block)
        params[:erb] = src
        dynamic_element(name, params, HtmlElementSpec, &block)
      end

      def method_missing(sym, *args, &block)
        case sym
        when :grid_form, :fomr_element, :simple_table
          super
        else
          if Core::SpecOptionMeta::get_option_meta(sym)
            super
          else
            define_html_element(sym, *args, &block)
          end
        end
      end
      
      def define_html_element(sym, *args, &block)
        name = args[0]
        params = args[1] || {}
        case name
        when Symbol, nil
          params[:html_tag] = sym
          params[:dummy_element] = true unless name
          dynamic_element(name, params, HtmlElementSpec, &block)
        else
          params[:html_tag] = sym
          params[:static_data] = name
          #params[:dummy_element] = true 
          dynamic_element(nil, params, HtmlElementSpec, &block)
        end
      end
    end

    class LinkElementSpec < HtmlElementSpec
      def make_default_template
        "<a id='#{sym}' />"
      end
    end

    class TableElementSpec < HtmlElementSpec
       def make_default_template
        opt = @option[:table_opt]
        classes = (opt[:classes]||[nil])
        opt.delete(:name)
        opt.delete(:classes)
        t = (@option[:meta_template] || TDL::meta_template(:table))
        t.expand(ret="") do |m|
          m.table(opt) do |mt|
            mt.header do |mh|
              children.each do |c|
                mh.item(c.sym)
              end
            end
            classes.each do |cls|
              h = { :id=>sym }
              h[:class] = cls if cls
              mt.detail(h) do |md|
                children.each do |c|
                  opt = (c.option[:table_opt] || {})
                  opt[:id] = c.sym
                  md.item(SanitizedString[c.make_default_template], opt)
                end
              end
            end
          end
        end
        ret
      end
      
      def column(sym, opt={}, &block)
        dynamic_element(sym, opt, HtmlElementSpec, &block)
      end
    end


    Core::SpecOptionMeta::define_option(:html_tag)
    Core::SpecOptionMeta::define_option(:html_attr)
    Core::SpecOptionMeta::define_option(:static_data)
    Core::SpecOptionMeta::define_option(:table_opt)

    module InputField 
       include Extention::ValueAttr
       extend Core::SpecOptionMeta::CompilerOption
       private

       def complie_for_single(element, dyn_spec, cg, value_attr)
         gen_element_with_attr(element, dyn_spec, cg)
         cg.code("e.attributes['#{value_attr}'] = val if val")
         cg.code('e.attributes["type"] = ' + dyn_spec.option[:field_type].inspect)
         cg.code("get_mainstream << e.to_s")
       end
       Core::SpecOptionMeta::define_option(:input_field, self)
     end

    class FieldMatcher
      def initialize(name)
        @name = name.to_s
      end

      def match(e)
        e.attributes['name'] == @name
      end
      
    end

    module FormMeta
      include Core::SpecOptionMeta::MetaData
      def define_element_ext(spec, name, sym, params, &block)
        pp = params.clone 
        pp[name] = true
        pp[:html_tag] = :form
        spec.dynamic_element(sym, pp, HtmlElementSpec, &block)
      end
      Core::SpecOptionMeta::define_option(:form_element, self)
    end

    module GridForm
      include Core::SpecOptionMeta::MetaData
      def define_element_ext(spec, name, sym, params, &block)
        pp = params.clone 
        pp[:replace_attr] = true
        pp[name] = true
        spec.dynamic_element(sym, pp, GridFormElementSpec, &block)
      end
      Core::SpecOptionMeta::define_option(:grid_form, self)
    end

    class GridFormElementSpec < HtmlElementSpec
      def make_default_template
        t = (@option[:meta_template] || TDL::meta_template(:grid_form))
        table_opt = @option[:table_opt]
        buttons = (@option[:buttons] || [ :ok ])
        form_attr = {}
        form_attr[:id] = sym
        form_attr[:method] = (@option[:method] || 'post')
        form_attr[:action] = (@option[:action] || '')

        t.expand(ret="") do |m|
          m.form(form_attr) do |mf|
            mf.table(table_opt) do |mt|
              children.each do |c|
                mt.tr do |mtr|
                  mtr.description(c.option[:description] || c.sym)
                  mtr.item(SanitizedString[c.make_default_template])
                end
              end
              b = buttons.collect do |b|
                "<input type='submit' name='#{b}' value='#{b}' /> "
              end.join
              mt.buttons(SanitizedString[b])
            end
          end
        end
        #print ret
        ret
      end
      Core::SpecOptionMeta::define_option(:method)
      Core::SpecOptionMeta::define_option(:action)
      Core::SpecOptionMeta::define_option(:buttons)
    end

    class SelectionElementSpec < Core::DynamicElementSpec
      def opt(sym, val=nil)
        e = REXML::Element.new('option')
        e.attributes['value'] = sym.to_s
        e.text = (val or sym.to_s)
        @option[:selection_element][1] << e
      end
    end

    class RadioButtonsElementSpec < Core::DynamicElementSpec
      def opt(sym, val=nil)
        e = REXML::Element.new('input')
        e.attributes['name'] = self.sym.to_s
        e.attributes['type'] = 'radio'
        e.attributes['value'] = sym.to_s
        e.text = (val or sym.to_s)
        @option[:selection_element][1] << e
      end
    end

    Core::SpecOptionMeta::define_option(:field_type)
    Core::SpecOptionMeta::define_option(:description)

    MetaTemplate = {}

    def self.meta_template(name)
      ret = MetaTemplate[name]
      return ret if ret
      case name
      when :html
        text = unindent_auto(<<-END)
        <html>
        <head>
          <title>title</title>
          <link rel='stylesheet' type='text/css' />
        </head>
        <body />
        </html>
        END
         
        ret = TemplateText.new(text) do
          Amrita2::define_template_spec do 
            dynamic_element(:head, :matcher=>HtmlTag::HEAD) do 
              dynamic_element(:title, :matcher=>HtmlTag::TITLE) 
              dynamic_element(:css, :matcher=>HtmlTag::LINK, :value_attr=>:href) 
            end
            dynamic_element(:body, :matcher=>HtmlTag::BODY)
          end
        end
        ret.setup
      when :table
        text = unindent_auto(<<-END)
        <table>
          <tr><th /></tr>
          <tr><td /></tr>
        </table>
        END
         
        ret = TemplateText.new(text) do
          Amrita2::define_template_spec do 
            dynamic_element(:table, :matcher=>HtmlTag::TABLE, :replace_attr=>true) do 
              dynamic_element(:header, :matcher=>HtmlTag::TR)  do 
                dynamic_element(:item, :matcher=>HtmlTag::TH)  
              end
              dynamic_element(:detail, :matcher=>HtmlTag::TR, :replace_attr=>true)  do 
                dynamic_element(:item, :matcher=>HtmlTag::TD, :replace_attr=>true)  
              end
            end
          end
        end
        ret.setup
      when :grid_form
        text = unindent_auto(<<-END)
        <form id='form' method='post'>
        <table>
          <tr><th /><td /></tr>
          <tr><td colspan='2'><div align='right'><span id='buttons' /></div></td></tr>
        </table>
        </form>
        END
         
        ret = TemplateText.new(text) do
          Amrita2::define_template_spec do 
            dynamic_element(:form, :matcher=>HtmlTag::FORM, :replace_attr=>true) do 
              dynamic_element(:table, :matcher=>HtmlTag::TABLE, :replace_attr=>true) do 
                dynamic_element(:tr, :matcher=>HtmlTag::TR, :replace_attr=>true)  do 
                  dynamic_element(:description, :matcher=>HtmlTag::TH)  
                  dynamic_element(:item, :matcher=>HtmlTag::TD, :replace_attr=>true)  
                end
                dynamic_element(:buttons)
              end
            end
          end
        end
        ret.setup
      else
        raise "unknown meta template #{name}"
      end
      MetaTemplate[name] = ret
      ret
    end

    module Component
      extend Core::SpecOptionMeta::CompilerOption
      extend Core::SpecOptionMeta::ElementExt
      def check_option(spec, k, v)
        super
      end

      def define_element_ext(spec, name, sym, params, &block)
        super
      end

      def make_default_template
        super
      end

      def code_for_element_and_scalar(element, dyn_spec, cg)
        super
      end

      def code_for_not_leaf_element(element, dyn_spec, cg, &block)
        super
      end

      def generate_expander(spec, mod, parent=nil)
        super
      end

      def self.define_component(name, mod)
        Core::SpecOptionMeta::define_option(name, mod)
      end
    end

    module Calender
      include Core::SpecOptionMeta::MetaData
      def define_element_ext(spec, name, sym, params, &block)
        spec.dynamic_element(sym, params, CalenderElementSpec, &block)
      end
      Core::SpecOptionMeta::define_option(:simple_calender, self)
      
      class CalenderElementSpec < HtmlElementSpec
        def initialize(sym, parent, option={}, &block)
          super
          make_calender_table if block
        end

        def make_calender_table
          @in_setup = true
          mp = @month_title_proc
          dp = @date_proc
          self.instance_eval do 
            div(:calender, :delete_id=>true) do
              table do
                tr do
                  th(:header, :html_attr=> { :colspan=>7} ) do 
                    span(:month)
                    instance_eval(&mp) if mp
                  end
                end
                tr do 
                  th(:week_title)
                end
                tr(:week) do
                  td(:day, :html_attr=> { :align=>'right'} ) do
                    span(:default_day, :delete_tag=>true)
                    instance_eval(&dp) if dp
                  end
                end
              end
            end
          end
        ensure
          @in_setup = false
        end

        def month_title(&block)
          @month_title_proc = block
        end

        def cal_date(&block)
          @date_proc = block
        end
        
        def generate_expander(mod, parent=nil)
          Expander.new(parent, self, mod)
        end
      end

      class Expander < Expander::Node
        def expand(data, i=0)
          raise unless data.kind_of?(Data)
          @method.call do |m|
            m.calender do |mc|
              data.expand_calender(self, mc)
            end
          end
        end
      end

      class Data
        attr_accessor :days_of_week
        def initialize(y,m, &block)
          @y, @m = y, m
          @date_proc = block
          @days_of_week = %w(Sun Mon Tue Wed Thu Fri Sat)
          @first_date = Date.new(y,m,1)
          @first_wday = @first_date.wday
          @last_date = @first_date + 28
          @last_date = @last_date.succ while @last_date.month == @first_date.month
          @last_date -= 1
          @days_of_this_month = Integer(@last_date - @first_date + 1)
        end

        def expand_calender(expander, m)
          m.header do |mh|
            expand_header(mh)
          end
          days_of_week.each do |w|
            m.week_title(w)
          end

          6.times do |w|
            m.week do |mw|
              expand_week(w, mw, expander)
            end
          end
        end

        def expand_header(m)
          m.month("#{@y}/#{@m}")
        end

        def expand_week(w, m, expander)
          (0..6).each do |d|
            day = get_day(w, d)
            if day
              expand_day(Date.new(@y,@m,day),m ,expander)
            else
              m.day(day) do 
              end
            end
          end
        end

        def expand_day(date, m, expander)
          d_data = (@date_proc and @date_proc.call(date))
          if d_data
            #exp = expander.spec.day_spec.generate_expander(expander)
            exp = get_day_expander(expander)
            exp.expand(d_data)
          else
            m.day do |md|
              md.default_day(date.day)
            end
          end
        end

        def get_day_expander(expander)
          #p expander.name
          #p expander.children[0].name
          expander.children[0].children[2].children[0]
        end

        def get_day(w, d)
          ret = w*7 + d - @first_wday + 1
          ret = nil if ret < 1 or ret > @days_of_this_month
          ret
        end
      end
    end
  end
end
