require 'amrita2/core'

module Amrita2
  module Extention
    module CheckTrue
      extend Core::SpecOptionMeta::CompilerOption
      def code_for_leaf_element(element, dyn_spec, cg, &block)
        sym = dyn_spec.sym
        cg.define_element_method(sym, dyn_spec.output_stream) do
          cg.if_("val == true") do
            cg.put_constant(element.to_s)
            cg.else_ do
              code_for_element_and_scalar(element, dyn_spec, cg)
            end
          end
        end
      end
      
      Core::SpecOptionMeta::define_option(:check_true, self)
    end

    module UseOriginalElement
      extend Core::SpecOptionMeta::CompilerOption
      include Amrita2::Runtime

      def self.check_option(spec, k, v)
        spec.option[:check_true] = spec.option[:replace_attr] = false
      end

      def code_for_leaf_element(element, dyn_spec, cg, &block)
        sym = dyn_spec.sym
        cg.define_element_method(sym, dyn_spec.output_stream) do
          cg.if_("block_given?") do
            gen_element_with_attr(element, dyn_spec, cg)
            cg.code("e.text = #{element.text.inspect}")
            cg.code("get_mainstream << yield(e).to_s")
            cg.else_ do
              code_for_element_and_scalar(element, dyn_spec, cg)
            end
          end
        end
      end

      def code_for_element_with_child(element, dyn_spec, cg)
        cg.code("yield(#{dyn_spec.module_name})")
        cg.if_("get_substream(:#{dyn_spec.output_stream}).size == 0") do
          cg.generate_static_element_code(element, dyn_spec.option) do 
            element.each_child do |e|
              e.amrita_compile_main_code(cg)
            end
          end
          cg.else_ do
            cg.code("output_substream(:#{dyn_spec.name_for(element)})")
            element.each_child do |e|
              e.amrita_compile_main_code(cg)
            end
            cg.put_constant(end_tag(element))
          end
        end
      end

      def code_for_not_leaf_element(element, dyn_spec, cg, &block)
        if dyn_spec.matched_many? and dyn_spec.first_match?(element)
          make_selection_code(element, dyn_spec, cg, &block)
        end
        sym = dyn_spec.name_for(element)
        cg.define_element_method(sym, dyn_spec.output_stream) do
          code_for_element_with_child(element, dyn_spec, cg, &block)
        end
        
        if ! dyn_spec.matched_many? or dyn_spec.first_match?(element)
          cg.define_module(dyn_spec.module_name) do
            cg.code("Methods = #{dyn_spec.get_methods.inspect}")
            cg.code("extend Amrita2::Core::Runtime")
            cg.define_method(sym, "val=nil", "attrs={}") do
              cg.new_context("get_substream(:#{sym})") do
                cg.code("e = yield(new_element(#{element.name.inspect}))") 
                cg.code("get_mainstream << start_tag(e)")
              end
              yield
            end
            cg.code("module_function :#{sym}")
          end
        end
      end

      Core::SpecOptionMeta::define_option(:use_original_element, self)
      public
      class Expander < Expander::Leaf
        def expand(data, i=nil)
          case data
          when Proc
            @method.call do |e|
              data.call(e)
            end
          else
            super
          end
        end
      end
      public
      def generate_expander(spec, mod, parent=nil)
        Expander.new(parent, spec, mod)
      end
    end

    module ExpandByMember
      extend Core::SpecOptionMeta::CompilerOption
      def self.check_option(spec, k, v)
        spec.option[:check_true] = spec.option[:replace_attr] = false
      end
      def code_for_not_leaf_element(element, dyn_spec, cg, &block)
        if dyn_spec.matched_many? and dyn_spec.first_match?(element)
          make_selection_code(element, dyn_spec, cg, &block)
        end
        sym = dyn_spec.name_for(element)
        if ! dyn_spec.matched_many? or dyn_spec.first_match?(element)
          cg.define_element_method(sym, dyn_spec.output_stream) do
            code_for_element_with_child(element, dyn_spec, cg, &block)
          end
          cg.define_module(dyn_spec.module_name) do
            cg.code("Methods = #{dyn_spec.get_methods.inspect}")
            cg.code("extend Amrita2::Runtime")
            yield
          end
        end
      end

      def code_for_element_with_child(element, dyn_spec, cg)
        cg.generate_static_element_code(element, dyn_spec.option) do 
          cg.if_("block_given?") do
            cg.code("yield(#{dyn_spec.module_name})")
            cg.else_ do
              dyn_spec.children.each do |c|
                cg.code("#{dyn_spec.module_name}::#{c.sym}(val.#{c.sym})")
              end
            end
          end
          element.each_child do |e|
            e.amrita_compile_main_code(cg)
          end
        end
      end
      Core::SpecOptionMeta::define_option(:expand_by_member, self)

      class Expander < Expander::Node
        def expand(data, i=0)
          if data.kind_of?(Enumerable)
            data.each_with_index do |d, i|
              expand(d, i)
            end
          else
            @method.call(i) do 
              each do |c|
                dd = data.send(c.name)
                c.expand(dd)
              end
            end
          end
        end
      end

      public
      def generate_expander(spec, mod, parent=nil)
        Expander.new(parent, spec, mod)
      end
    end

    module UseArgs
      extend Core::SpecOptionMeta::CompilerOption
      def self.check_option(spec, k, v)
        spec.option[:check_true] = spec.option[:replace_attr] = false
      end
      def code_for_leaf_element(element, dyn_spec, cg, &block)
        cg.code('include Amrita2::Runtime')
        sym = dyn_spec.sym
        cg.define_method(sym, "*args", "&block") do
          cg.new_context("get_substream(:#{dyn_spec.output_stream})") do
            complie_for_args(element, cg, dyn_spec.option[:delete_id])
          end
        end
        cg.code("module_function :#{sym}")
      end

      private

      def complie_for_args(element, cg, delete_id)
        replace_args(element)
        #cg.code("get_mainstream << \"#{element.to_s}\"")
        s = ""
        element.write(s, -1, false, cg.ie_hack)
        cg.code("get_mainstream << \"#{s}\"")
      end

      def replace_args(element)
        element.delete_attribute('id')
        element.attributes.each do|k,v|
          replace_attr_args(v)
        end
        element.each_child do |c|
          case c
          when REXML::Text
            c.value = replace_text_args(c.value) if c.value
          when REXML::Element
            replace_args(c)
          end
        end
      end

      def replace_attr_args(s)
        s.gsub!(/\$(\d)/) do |ss|
          case $1
          when "0"
            '#{block.call.amrita_sanitize_as_attribute if block}'
          else
            '#{args[' + ($1.to_i-1).to_s + '].amrita_sanitize_as_attribute}'
          end
        end
      end

      def replace_text_args(s)
        s.gsub(/\$(\d)/) do |ss|
          case $1
          when "0"
            '#{block.call.amrita_sanitize if block}'
          else
            '#{args[' + ($1.to_i-1).to_s + '].amrita_sanitize}'
          end
        end
      end

      Core::SpecOptionMeta::define_option(:use_args, self)
      public
      def generate_expander(spec, mod, parent=nil)
        Expander::TupleExpander.new(parent, spec, mod)
      end

    end

    module ValueAttr
      extend Core::SpecOptionMeta::CompilerOption
      def self.check_option(spec, k, v)
        spec.option[:check_true] = spec.option[:replace_attr] = false
      end
      def code_for_leaf_element(element, dyn_spec, cg, &block)
        cg.code('include Amrita2::Runtime')
        sym = dyn_spec.sym
        cg.define_element_method(sym, dyn_spec.output_stream) do
          complie_for_single(element, dyn_spec, cg, dyn_spec.option[:value_attr])
        end
      end

      def generate_expander(spec, mod, parent=nil)
        ret = super
        def ret.expand(data)
          case data
          when false, nil, "false"
            @method.call(nil)
          else
            super
          end
        end
        ret
      end

      private

      def complie_for_single(element, dyn_spec, cg, value_attr)
        gen_element_with_attr(element, dyn_spec, cg)
        cg.code("e.attributes['#{value_attr}'] = val")
        cg.code("e.text = #{element.text.inspect}") if element.text
        cg.code("get_mainstream << e.to_s")
      end

      Core::SpecOptionMeta::define_option(:value_attr, self)
    end

    module LinkAttr
      extend Core::SpecOptionMeta::CompilerOption
      def self.check_option(spec, k, v)
        spec.option[:check_true] = spec.option[:replace_attr] = false
      end
      def code_for_leaf_element(element, dyn_spec, cg, &block)
        cg.code('include Amrita2::Runtime')
        sym = dyn_spec.sym
        cg.define_method(sym, "href", "val=nil", "attrs={}") do 
          cg.if_("val.kind_of?(Hash)") do
            cg.code("attrs = val")
            cg.code("val = nil")
          end
          cg.new_context("get_substream(:#{dyn_spec.output_stream})") do
            complie_for_link(element, dyn_spec, cg, dyn_spec.option[:link_attr])
          end
        end
        cg.code("module_function :#{sym}")
      end

      private

      def complie_for_link(element, dyn_spec, cg, link_attr)
        href, checks = *link_attr
        gen_element_with_attr(element, dyn_spec, cg)
        checks.each do |r|
          cg.code("href = '' if #{r.inspect} =~ href")
        end
        cg.code("e.attributes['#{href}'] = href if href")
        cg.code("e.delete_attribute('#{dyn_spec.option[:key_attr_name]}')") if dyn_spec.option[:delete_id]
        cg.code("e.text = (val || #{element.text.inspect}) ")
        cg.code("get_mainstream << e.to_s")
      end

      Core::SpecOptionMeta::define_option(:link_attr, self)
      class LinkExpander < Expander::TupleExpander
        def expand(data, i=nil)
          case data
          when Array
            case data.first
            when Array, AttrArray, Hash
              data.each_with_index do |d, i|
                expand(d, i)
              end
            else
              @method.call(*data)
            end
          when AttrArray
            @method.call(data[:href], data.body, data)
          else 
            super
          end
        end
      end

      public
      def generate_expander(spec, mod, parent=nil)
        LinkExpander.new(parent, spec, mod)
      end
    end

    module SelectionElement
      extend Core::SpecOptionMeta::CompilerOption
      def self.check_option(spec, k, v)
        spec.option[:check_true] = spec.option[:replace_attr] = false
      end
      def code_for_leaf_element(element, dyn_spec, cg, &block)
        cg.code('include Amrita2::Runtime')
        sym = dyn_spec.sym
        cg.define_element_method(sym, dyn_spec.output_stream) do
          complie_for_selection(element, dyn_spec, cg, dyn_spec.option[:selection_element])
        end
      end

      private

      def complie_for_selection(element, dyn_spec, cg, opts)
        select_attr, children = *opts
        children = element.elements  if element.elements.size > 0
        gen_element_with_attr(element, dyn_spec, cg)
        children.each do |c|
          sym = c.attributes["value"].intern
          cg.code("a = #{c.attributes.inspect}")
          cg.code("a[#{select_attr.inspect}] = '#{select_attr.to_s}' if val and val.to_s.size > 0 and val.to_s.intern == #{sym.inspect} ")
          cg.code("c = new_element(#{c.name.intern.inspect}, a)")
          cg.code("c.text = #{c.text.inspect}") if c.text
          cg.code("e.add(c)")
        end
        cg.code("get_mainstream << e.to_s")
      end

      Core::SpecOptionMeta::define_option(:selection_element, self)
    end

    module ERBElement
      extend Core::SpecOptionMeta::CompilerOption
      def self.check_option(spec, k, v)
        spec.option[:check_true] = spec.option[:replace_attr] = false
      end

      def code_for_leaf_element(element, dyn_spec, cg, &block)
        unless dyn_spec.option[:erb].kind_of?(String)
          dyn_spec.option[:erb] = get_source(element)
        end
        require 'erb'
        cg.code("require 'erb'")
        cg.code('include Amrita2::Runtime')
        if dyn_spec.option[:use_erb_args]
          use_args(element, dyn_spec, cg, &block)
        elsif dyn_spec.option[:use_single_arg]
          comiple_erb(element, dyn_spec, cg, &block)
        else
          use_binding(element, dyn_spec, cg, &block)
        end
      end

      private

      def get_source(element)
        ret = ""
        element.each_child do |c|
          case c
          when REXML::Text
            ret << c.to_s
          when REXML::CData
            ret << c.text
          end
        end
        ret
      end

      def comiple_erb(element, dyn_spec, cg, &block)
        sym = dyn_spec.sym
        erb_method_name = "_#{sym}_erb"
        cg.define_method(erb_method_name, "args") do 
          cg.code("$_ = args")
          src = dyn_spec.option[:erb]
          cg.code(ERB.new(src.inspect).src)
          cg.code("eval _erbout")
        end
        cg.code("module_function :#{erb_method_name}")

        cg.define_method(sym, "*_args", "&block") do
          cg.new_context("get_substream(:#{dyn_spec.output_stream})") do
            cg.code("get_mainstream << #{erb_method_name}(*_args, &block)")
          end
        end
        cg.code("module_function :#{sym}")
      end

      def use_binding(element, dyn_spec, cg, &block)
        cg.code('include Amrita2::Runtime')
        sym = dyn_spec.sym
        cg.define_method(sym, "_binding", "val=nil", "&block") do
          cg.new_context("get_substream(:#{dyn_spec.output_stream})") do
            src = dyn_spec.option[:erb]
            cg.code("return unless _binding")
            cg.code("current_context.data = val")
            src = "<% $_ = current_context.data %>" + src
            cg.code("get_mainstream << ERB.new(#{src.inspect}).result(_binding)")
          end
        end
        cg.code("module_function :#{sym}")
      end

      def use_args(element, dyn_spec, cg, &block)
        cg.code('include Amrita2::Runtime')
        sym = dyn_spec.sym
        erb_method_name = "_#{sym}_erb"
        cg.define_method(erb_method_name, "*_args") do 
          cg.code("$_ = _args")
          src = replace_args(dyn_spec.option[:erb])
          cg.code(ERB.new(src.inspect).src)
          cg.code("eval _erbout")
        end
        cg.code("module_function :#{erb_method_name}")

        cg.define_method(sym, "*_args", "&block") do
          cg.new_context("get_substream(:#{dyn_spec.output_stream})") do
            cg.code("get_mainstream << #{erb_method_name}(*_args, &block)")
          end
        end
        cg.code("module_function :#{sym}")
      end

      def replace_args(src)
        src.gsub(/\$(\d)/) do |ss|
          case $1
          when "0"
            '(yield if block_given?)'
          else
            '($_[' + ($1.to_i-1).to_s + '])'
          end
        end
      end

      Core::SpecOptionMeta::define_option(:use_single_arg)
      Core::SpecOptionMeta::define_option(:erb, self)
      Core::SpecOptionMeta::define_option(:use_erb_args)
      public
      def generate_expander(spec, mod, parent=nil)
        Expander::TupleExpander.new(parent, spec, mod)
      end
    end
  end
end

