require 'amrita2/core'
require 'amrita2/ext'

module Amrita2
  class Template
    include Amrita2
    include Core

    DefaultTypeMap = { 
      :use_args => { :use_args => true},
      :use_original_element=> { :use_original_element => true},
      :expand_by_member => { :expand_by_member => true },
      :value_attr => { :value_attr => true },
      :erb => { :erb => true },
      :link => { :link_attr => [:href, [/javascript:.*/]] },
      :input => { :value_attr => :value },
      :checkbox => { :value_attr => :checked },
      :select => { :selection_element => [:selected, []] },
    }

    # keep id attribute of output if set.
    attr_accessor :keep_id
    # The name of attribute that will be used for template expandion by amrita.
    # You will need to set this if you use _id_ attribute fom DOM.
    # For expample, if this was set to "amrita_id", 
    # you can use amrita_id for amrita and id for DOM.
    attr_accessor :amrita_id, :spec, :ie_hack, :debug_source

    attr_accessor :passive_po, :type_map

    attr_reader :mod

    def initialize(&block)
      @keep_id = false
      @amrita_id = nil
      @setuped = false
      @passive_po = false
      @spec = nil
      @spec_proc = block
      @type_map = DefaultTypeMap.clone
      @ie_hack = false
    end

    def setup
      return if @setuped
      rexml_doc = (get_document or generate_default_document)
      @root = rexml_doc.root
      @spec = get_spec
      @mod = @spec.create_template_module(rexml_doc, rexml_doc)
      @expander = get_expander
      self.extend @mod
      @setuped = true
    end

    def get_option
      ret = {
        :check_true=>true, 
        :replace_attr=>true,
        :delete_id => (not @keep_id), 
        :key_attr_name=>( @amrita_id or :id),
        :ie_hack => @ie_hack,
        :debug_source => @debug_source,
      }
      ret
    end

    def get_spec
      if @spec 
        @spec
      elsif @spec_proc
        @spec = @spec_proc.call
      else
        @spec = Amrita2::make_default_spec(@root, @type_map, get_option)
      end
      @spec = @spec.clone unless @spec.cloned
      @spec 
    end

    def get_expander
#       if passive_po
#         Expander.new(get_option)
#       else
#         ExpanderForActivePO.new(get_option)
#       end
      @spec.generate_expander(@mod)
    end

    def expand(out, data=nil, &block)
      setup unless @setuped
      if data
        @expander.expand_template(out, data)
      else
        expand_template(out, &block)
      end
    end

    def expand_parts(name, data=nil, &block)
      setup unless @setuped
      pexp = @expander.get_parts(name) || @expander.get_parts(name.intern) 
      if data
        pexp.expand_parts(data)
      else
        pexp.expand_parts(block)
      end
    end

    def prettyprint=(f)
      warn 'prettyprint is not supported'
    end

    def compact_space=(f)
      warn 'compact_space is not supported'
    end

    def escaped_id=(f)
      warn 'escaped_id is not supported'
    end

    def xml=(f)
      warn 'xml option is not supported'
    end

    def asxml=(f)
      warn 'asxml option is not supported'
    end

    def pre_format=(f)
      warn 'pre_format option is not supported'
    end

    def use_compiler=(f)
      warn 'use_compiler option is not supported'
    end

    def get_document
      nil
    end

    def generate_default_document
      src = get_spec.make_default_template
      #STDOUT.puts src
      ret = REXML::Document.new  src
      ret
    rescue
      puts src
      raise
    end
  end

  class TemplateFile < Template
    def initialize(fname, &block)
      super(&block)
      @fname = fname
    end

    def get_document
      REXML::Document.new(File.new(@fname))
    end
  end

  class TemplateText < Template
    def initialize(text, &block)
      super(&block)
      @text = text
    end

    def get_document
      REXML::Document.new(@text)
    end
  end
end
