#!/usr/bin/env ruby
$: << '/home/tnaka/amrita2/lib'

require 'cgi'
require 'amrita2/template'
require 'amrita2/generator'

DATAFILE_PATH="bookmark.dat"

class Item
  attr_reader :group, :name, :url

  def initialize(group, name, url)
    @group, @name, @url = group, name, url
  end

  def to_s
    %Q[#{@group}|#{@name}|#{@url}]
  end

  def link(m)
    m.link do |ml|
      ml.url(@url, :href=>@url)
    end
  end
end

class BookmarkList
  attr_reader :groups

  def initialize
    @groups = {}
  end

  def load_from_file(path)
    File::open(path) do |f|
      f.each do |line|
        begin
          add_new_item(*line.chomp.split('|'))
        rescue
        end
      end
    end
  end

  def save_to_file(path)
    File::open(path, "w") do |f|
      @groups.each do |k, v|
        v.each do |data|
          f.puts data.to_s
        end
      end
    end
  end

  def add_new_item(group="", name="", url="", *x)
    item = Item.new(group, name, url)
    @groups[group] ||= []
    @groups[group] << item
  end
end


def template(groups)
  groups.delete_if { |g| g == ""}
  Amrita2::Template.new do
    Amrita2::define_template_spec do 
      html do
        title('Amrita2 bookmark sample')
        erb(:count, 'There are <%= $1 %> groups in this list now.')
        div(:groups) do 
          h2(:group_name)
          simple_table(:items, :table_opt=>{ :border=>1 }) do 
            column(:name)
            column(:link) do 
              a(:url, :replace_attr=>true)
            end
          end
        end
        hr
        grid_form(:form) do 
          select(:group) do |gsel|
            groups.each  do |g| 
              gsel.opt(g.intern, g) 
            end
          end
          text_field(:new_group)
          text_field(:title)
          text_field(:url)
        end
      end
    end
  end
end

def make_model_data(bm, selected_group, title, url)
  #p selected_group
  groups = bm.groups.keys.sort
  {
    :groups => groups.collect do |k|
      {
        :group_name=>k,
        :items=>bm.groups[k]
      }
    end ,
    :count=>groups.size,
    :form => proc do |m|
      m.form do |mf|
        mf.url(url)
        mf.title(title)
        mf.group(selected_group)
        mf.new_group("")
      end
    end
  }
end

def main
  bm = BookmarkList.new
  bm.load_from_file(DATAFILE_PATH)
  cgi = CGI.new
  url = cgi['url']
  group = ""
  if url and url.size > 0
    group = cgi['new_group']
    group = cgi['group'] if group == ""
    name = cgi['title']
    name = url if name == ""
    bm.add_new_item(group, name, url)
    bm.save_to_file(DATAFILE_PATH)
  end
  tmpl = template(bm.groups.keys.sort)
  puts cgi.header
  tmpl.expand($stdout, make_model_data(bm, group, name, url))
end

main
