require 'amrita2/template'
require 'amrita2/a2tdl'

require 'webrick'
require 'webrick/session'
class DialogServlet < WEBrick::HTTPServlet::AbstractServlet
  def self.get_instance(server, dialog, *options)
    self.new(server, dialog, *options)
  end

  def initialize(server, dialog, *options)
    super(server ,*options)
    @dialog_cls = dialog
  end

  def do_GET(req, res)
    session = WEBrick::Session.new(req, res, 'database_manager'=> WEBrick::Session::MemoryStore)
    dialog = session['dialog'] 
    unless dialog.kind_of?(@dialog_cls)
      session.delete
      session = WEBrick::Session.new(req, res, 'database_manager'=> WEBrick::Session::MemoryStore)
      dialog = @dialog_cls.new
    end
    if dialog.process(req,res, session)
      session['dialog'] = dialog
      session.update
    else
      session.delete
    end
  end

  alias do_POST do_GET

end

class Dialog
  include Amrita2
  class Screen
    include Amrita2
    attr_accessor :form_spec, :form_opt

    def initialize(dialog, sid, opt={}, &block)
      @dialog = dialog
      @sid = sid
      @proc = []
      title = opt[:title] || sid.to_s 
      form_spec = nil
      @form_opt = { 
        :table_opt=>{ :border=>1 }
      }
      me = self
      @template = Template.new do
        Amrita2::define_template_spec do
          html do 
            title(title)
            
            top = dialog.common_areas[:top]
            if top
              div { instance_eval(&top) }
            end
            me.form_spec = grid_form(:form, me.form_opt) do |s|
              s.instance_eval(&block)
            end
            hr
            dynamic_element(:msg)
            bottom = dialog.common_areas[:bottom]
            if bottom
              hr
              div { instance_eval(&bottom) }
            end
          end
        end
      end
    end

    def setup
      @form_opt[:buttons] = @commands = get_commands
      @template.setup
    end

    def process(sid, &block)
      @proc << [sid, block]
      self
    end

    def get_commands
      @proc.collect do |sid, block|
        sid
      end
    end


    def do_process(req, res, data)
      cmd = @commands.find do |c|
        req.query[c.to_s]
      end
      req.query.each do |k,v|
        data[k.intern] = v
      end
      @msg = ''
      @proc.each do |sid, block|
        if sid == cmd
          @dialog.data_to_iv(data)
          block.call(data)
          @dialog.iv_to_data(data)
          break 
        end
      end
    end

    def show(res, data)
      @form_spec.children.each do |c|
        data[c.sym] = data[c.sym] || ''
      end
      @template.expand(out="", :form => data, :msg=>@dialog.msg)
      res['content-type'] = 'text/html'
      res.body = out
      @sid
    end
  end

  attr_accessor :msg, :common_areas

  def initialize
    @screens = {}
    @common_areas = {}
    init_dialog
  end

  def define_dialog(&block)
    instance_eval(&block)
    setup
    self
  end

  def setup
    @screens.each do |k, s|
      s.setup
    end
  end

  def init_screen(opt={}, &block)
    screen(:init, opt, &block)
  end

  def screen(sid, opt={}, &block)
    @screens[sid] = Screen.new(self, sid, opt, &block)
  end


  def common_area(area_id, &block)
    @common_areas[area_id] = block
  end

  def next_screen(sid)
    @next_screen = sid
  end

  def message(msg)
    @msg = msg
  end

  def process(req, res, session)
    @sid = session['next'] || :init
    @current_screen = @screens[@sid]
    data = session['data'] || {}
    @next_screen = @sid
    @session_init = false
    @current_screen.do_process(req, res, data)
    @next_screen ||= :init
    data = { } if @session_init
    @screens[@next_screen].show(res, data)
    session['next'] = @next_screen
    session['data'] = data
    not @session_init
  end

  def init_session
    @msg = ''
    @next_screen = :init
    @session_init = true
  end
    SystemVariables = [:@template, :@form, :@form_spec, :@form_opt, :@dialog, :@proc, :@commands, :@sid, :@screens, :@next_screen, :@session_init, :@current_screen, :@msg]

  def data_variables
    instance_variables.collect do |v|
      v.intern
    end - SystemVariables
  end

  def iv_to_data(data)
    data_variables.each do |id|
      v = self.instance_variable_get(id)
      k = id.to_s[1..-1]
      next if k == ""
      data[k.intern] = v
    end
  end

  def data_to_iv(data)
    data.each do |k,v|
      id = '@' + k.to_s
      self.instance_variable_set(id, v)
    end
  end
end
