#!/usr/bin/env ruby

require 'webrick'

require 'amrita2/template'
require 'hello'
require 'number'
require 'order'
require 'filelist'

MapItem = Struct.new(:name, :cls)

MAP = [
  MapItem.new('hello', HelloDialog),
  MapItem.new('number', NumberGameDialog),
  MapItem.new('order', OrderDemoDialog),
  MapItem.new('filelist', FileListDialog)
]

def start_servlet(httpd)
  MAP.each do |s|
    httpd.mount("/#{s.name}", DialogServlet, s.cls)
  end
end


class IndexServlet < WEBrick::HTTPServlet::AbstractServlet
  include Amrita2

  def do_GET(req, res)
    if %r[/([a-z]*.rb).txt] =~ req.path
      File::open($1) do |f|
        res.body = f.read
      end
      res['content-type'] = 'text/plain'
    else
      res['content-type'] = 'text/html'
      res.body = index
    end
  end

  def index
    tmpl = Template.new do 
      Amrita2::define_template_spec do 
        html do 
          title('Amrita2 TDL and Dialog Demo')
          p('Amrita2 has a TDL(Template Definition Language) and Dialog Sysmtem.')
          p('This is a running Demo  of the two new feature.')
          simple_table(:servlet, :table_opt=> { :border=>1 }) do 
            column(:name) do 
              a(:link, :replace_attr=>true)
            end
            column(:src) do 
              a(:link, :replace_attr=>true)
            end
          end
          hr
          ul do 
            li do 
              a('see source of index servlet', :html_attr=>{ :href=>"/index.rb.txt"} )
            end
            li do 
              a('see source of dialog system', :html_attr=>{ :href=>"/dialog.rb.txt"} )
            end
          end
        end
      end
    end
    tmpl.expand(out="") do |m|
      MAP.each do |i|
        Kernel::p i
        m.servlet do |ms|
          ms.name do |mn|
            mn.link(i.name, :href=>"/#{i.name}")
          end
          ms.src do |mn|
            mn.link('see source', :href=>"/#{i.name}.rb.txt")
          end
        end
      end
    end
    out
  end
end

httpd = WEBrick::HTTPServer.new({:Port => 12345})
httpd.mount('/', IndexServlet)
start_servlet(httpd)
trap('INT') do
  httpd.shutdown
end
trap('TERM') do
  httpd.shutdown
end
httpd.start
