
require 'amrita2/core'
require 'amrita2/a2tdl'

module TodoHelper
  include Amrita2
  def list_setup_template(template)
    Amrita2::TemplateText.new(template) do
      Amrita2::define_template_spec do 
        html do 
          div(:not_done) do 
            dynamic_element(:item_done, :replace_attr=>true)
            dynamic_element(:description)
            a_link(:edit_link)
            a_link(:destroy_link)
          end
          div(:done) do 
            dynamic_element(:item_done, :replace_attr=>true)
            dynamic_element(:description)
            a_link(:edit_link)
            a_link(:destroy_link)
          end
        end
      end
    end
  end

  def display_item(item)
    { 
      # normal 'dynamic_element' accepts text
      :description=>item.description,
      # 'check_box' accepts Boolean value
      :item_done=>a(:onclick => %[document.location.href="/todo/toggle_check/#{item.id}";]),
      # 'a_link' accepts REXML::Element and a(..) { ... } is shortcut for it
      :edit_link=>a(:href=>"/todo/edit/#{item.id}") { "Edit" },
      # another way for setting attributes dynamically
      :destroy_link => proc do |m|
        msg = "Are you sure you want to delete this entry: #{item.description}"
        onclick=Amrita2::SanitizedString[%[return confirm("#{msg}");]]
        #onclick=''
        m.destroy_link("Destroy", "/todo/destroy/#{item.id}", :onclick=>onclick)
      end
    }
  end

  def list_po
    ret = { 
      :done => controller.done.collect do |r|
        display_item(r)
      end,
      :not_done => controller.not_done.collect do |r|
        display_item(r)
      end,
    }

    #STDOUT.puts ret.inspect
    ret
  end

=begin


# If spec has many iteration, it can be 'sub-routine' of spec like this

  def item_spec(s) # s is parent spec
    s.check_box(:item_done)
    s.dynamic_element(:description)
    s.a_link(:edit_link)
    s.a_link(:destroy_link)
  end

  def list_spec
    # get method as a Proc object
    item_spec = method(:item_spec)
    ret = Amrita2::define_template_spec do 
      html do 
      div(:not_done) do |s|
        item_spec.call(s) # call a Proc
      end
      div(:done) do |s|
        item_spec.call(s)
      end
      end
    end
    #STDOUT.puts ret.inspect
    ret
  end
=end
end

