class TodoController < ApplicationController
  include Amrita2
  model :todo
  scaffold :todo

  # done and not_done were added to the original controller
  attr_reader :done, :not_done

  def list
    @not_done = Todo.find_not_done
    @done = Todo.find_done
    @todo_list = list_po
    @form = binding
  end

  def add_item 
    #render_text @params.inspect
    item = Todo.new  # Create a new instance of Todo, so create a new item
    item.attributes = @params["new_item"]  # The fields of item should be set to what's in the "new_item" hash
    if item.save  # Try to save our item into the database
      redirect_to(:action => "list")  # Return to the list page if it suceeds
    else
      render_text "Couldn't add new item"  # Print an error message otherwise
    end
  end

  def destroy
    item = Todo.find(@params["id"])
    begin
      item.destroy
      redirect_to(:action => "list")
    rescue ActiveRecord::ActiveRecordError
      render_text "Couldn't destroy item" 
    end
  end

  def toggle_check
    item = Todo.find(@params["id"])

    # change the done flag.  ActiveRecord
    # will automatically do the boolean/int
    # conversion.  
    item.done = !item.done?

    if item.save
      redirect_to(:action => "list")
    else
      render_text "Couldn't update item" 
    end
  end

  def item_po(r)
    confirm_onclick = %[return confirm("Are you sure you want to delete this entry: #{r.description}")]
    { 
      :description=>r.description,
      :item_done=>a(:onclick => %[document.location.href="/todo/toggle_check/#{r.id}"]),
      :edit_link=>a(:href=>"/todo/edit/#{r.id}") { "Edit" },
      :destroy_link=>a(:href=>"/todo/destroy/#{r.id}",
                       :onclick=>confirm_onclick) { "Destory" },
    }
  end

  # Presentation Object for 'list' method
  # The result of this method will be used by Amrita2 for tempalte expansion
  def list_po
    {
      :done => Todo.find_done.collect do |r|
        item_po(r)
      end,
      :not_done => Todo.find_not_done.collect do |r|
        item_po(r)
      end
    }
  end

  # Presentation Object for Todo Record
  class TodoPO
    def initialize(r)
      @r = r
    end

    # passive PO method: 
    # The result of this method will be inserted in HTML output.
    def description
      @r.description
    end

    # active PO method:
    # m is a template Module with method for dynamic element.
    # If a method has a parameter for it, one of methods of that module
    # should be called by PO object.
    def item_done(m)
      m.item_done(nil, :onclick => %[document.location.href="/todo/toggle_check/#{@r.id}"])
    end

    def edit_link(m)
      m.edit_link("Edit", :href=>"/todo/edit/#{@r.id}")
    end

    def destroy_link(m)
      m.destroy_link("Destroy", :href=>"/todo/destroy/#{@r.id}",
                     :onclick=>%[return confirm("Are you sure you want to delete this entry: #{description}")])
    end
  end
end


