
require 'test/unit'
require 'test/unit/testsuite'
require 'test/unit/ui/console/testrunner'
require 'amrita2'
require 'rexml/document'

# ruby -I~/amrita2/lib ~/amrita2/test/test_samples.rb   |ruby -pe '$_.sub!(/^ */, "") ' 

module Amrita2SampleTest
  include Amrita2
  include Amrita2::Core
  
  PRJ_DIR = File::expand_path("~/amrita2")

  def get_result(dir, fname, out='')
    Dir::chdir("#{PRJ_DIR}/#{dir}") do
      IO.popen("echo #{out} | ruby -I../../lib #{fname}") do |f|
        doc = REXML::Document.new(f.read)
        doc.root
      end
    end
  end

  def get_texts(element, xpath)
    element.elements.to_a(xpath).collect do |e|
      e.text
    end
  end

  class TestHello < Test::Unit::TestCase
    def test_step1
      r = get_result("sample/hello/", "step1.rb")
      assert_equal('hello world', r.elements['body/h1'].text)
      assert_equal('Amrita2 is a html template libraly for Ruby', r.elements['body/p'].text)
    end

    def test_step2
      r = get_result("sample/hello/", "step2.rb")
      assert_equal(%w(1 2 3), get_texts(r, 'body/span'))
      assert_equal(%w(4 5 6 23 24), get_texts(r, 'body/p/span'))
    end

    def test_step4
      r = get_result("sample/hello/", "step4.rb")
      assert_equal("passive PO method", get_texts(r, 'body/span')[0])
      assert_equal('active PO method can replace attrs', r.elements['body/a'].text)
      assert_equal('http://www.ruby-lang.org/', r.elements['body/a'].attributes['href'])
      assert_equal('output in a Proc', r.elements['body/div'].text)
      assert_equal("active PO method can call dynamic element method many times", get_texts(r, 'body/p/span').join)
    end

    def test_hello
      r = get_result("sample/hello/", "hello.rb")
      assert_equal('hello world', r.elements['body/h1'].text)
      assert_equal('Amrita2 is a html template libraly for Ruby', r.elements['body/p'].text)
    end

    def test_hello2
      r = get_result("sample/hello/", "hello2.rb")
      assert_equal('hello world', r.elements['body/h1'].text)
      assert_equal('Amrita2 is a html template libraly for Ruby', r.elements['body/p'].text)
    end

    def test_list
      r = get_result("sample/hello/", "list.rb")
      assert_equal(%w(1 2 3), get_texts(r, 'li'))
    end

    def test_list2
      r = get_result("sample/hello/", "list2.rb")
      assert_equal(%w(1 2 3), get_texts(r, 'li'))
    end

    def test_dl
      r = get_result("sample/hello/", "dl.rb")
      assert_equal(["amrita", "Amrita2"], get_texts(r, 'dl/dt'))
      assert_equal(["amrita", "a template library", "Amrita2", "a better template library"], get_texts(r, 'dl/*'))
    end

    def test_table
      r = get_result("sample/hello/", "table.rb")
      assert_equal(["Ruby", "matz", "perl", "Larry Wall", "python", "Guido van Rossum"], get_texts(r, 'tr/td'))
    end

    def test_table2
      r = get_result("sample/hello/", "table2.rb")
      assert_equal(["Ruby", "matz", "perl", "Larry Wall", "python", "Guido van Rossum"], get_texts(r, 'tr/td'))
    end

    def test_table3
      r = get_result("sample/hello/", "table3.rb")
      assert_equal(["Ruby", "matz", "perl", "Larry Wall", "python", "Guido van Rossum"], get_texts(r, 'tr/td'))
    end

    def test_conditional
      r = get_result("sample/hello/", "conditional.rb")
      assert_equal(["Group A", "Group B", "Group C"], get_texts(r, 'body/div/h1'))
      assert_equal(["only_one"], get_texts(r, 'body/div/div/span'))
      assert_equal(%w(one two three), get_texts(r, 'body/div/div/ul/li'))
      assert_equal("This group has no data.", get_texts(r, 'body/div/div/em')[0])
    end

    def test_conditional2
      r = get_result("sample/hello/", "conditional2.rb")
      assert_equal(["Group A", "Group B", "Group C"], get_texts(r, 'body/div/h1'))
      assert_equal(["only_one"], get_texts(r, 'body/div/div/span'))
      assert_equal(%w(one two three), get_texts(r, 'body/div/div/ul/li'))
      assert_equal("This group has no data.", get_texts(r, 'body/div/div/em')[0])
    end
  end

  class TestSpec < Test::Unit::TestCase
    def test_spec
      r = get_result("sample/spec/", "spec.rb")
      assert_equal('hello world', r.elements['body/h1'].text)
      assert_equal('Amrita2 is a html template libraly for Ruby', r.elements['body/p'].text)
    end

    def test_matchers
      r = get_result("sample/spec/", "matcher.rb")
      assert_equal('hello world', r.elements['body/h1'].text)
      assert_equal('Amrita2 is a html template libraly for Ruby', r.elements['body/p'].text)
      assert_equal(%w(1 2 3), get_texts(r, 'body/ul/li'))
    end

    def test_attr
      r = get_result("sample/spec/", "attr.rb")
      assert_equal('replaced', r.elements['body/h1/a'].text)
      assert_equal('http://amrita2.rubyforge.org/', r.elements['body/h1/a'].attributes['href'])
      assert_equal('Ruby homepage(Japanese)', r.elements['body/p/a'].text)
      assert_equal('http://www.ruby-lang.org/ja/', r.elements['body/p/a'].attributes['href'])
    end

    def test_attrinline
      r = get_result("sample/spec/", "attrinline.rb")
      assert_equal('replaced', r.elements['body/h1/a'].text)
      assert_equal('http://amrita2.rubyforge.org/', r.elements['body/h1/a'].attributes['href'])
      assert_equal('Ruby homepage(Japanese)', r.elements['body/p/a'].text)
      assert_equal('http://www.ruby-lang.org/ja/', r.elements['body/p/a'].attributes['href'])
    end

    def test_inline
      r = get_result("sample/spec/", "inline.rb")
      a = r.elements['body/table/tr/td/a']
      assert_equal('http://www.ruby-lang.org/', a.attributes['href'])
      assert_equal('Ruby', a.text)
      a = r.elements['body/a']
      assert_equal('http://amrita2.rubyforge.org/', a.attributes['href'])
      assert(r.elements['body/small'])
    end

    def test_erb
      r = get_result("sample/spec/", "erb.rb")
      t = r.elements['head/title']
      assert_equal('Amrita2 with ERB ', t.text)
      t = r.elements['body/h1']
      assert_equal('Amrita2 with ERB  Sample', t.text)
      a = r.elements['body/table/tr/td/a']
      assert_equal('Ruby', a.text)
      assert_equal('http://www.ruby-lang.org/', a.attributes['href'])
    end
  end

  class TestCgi < Test::Unit::TestCase
    def test_sample
      r = get_result("sample/cgi/", "sample.cgi", 'name=a')
      assert_equal('Amrita2 CGI and HTML generator sample', r.elements['body/h1'].text)
      f = r.elements['body/form']
      assert_equal('post', f.attributes['method'])
      input = f.elements.to_a('table/tr/td/input')
      assert_equal('a', input[0].attributes['value'])
      assert_equal('name', input[0].attributes['name'])
      assert_equal('pass', input[1].attributes['name'])
      assert_equal('amrita2_user', input[2].attributes['name'])
      assert_equal('checkbox', input[2].attributes['type'])

      assert_equal('comment', f.elements['table/tr/td/textarea'].attributes['name'])
      assert_equal('name=a/', r.elements['body/span'].text)
    end

    def test_sample2
      r = get_result("sample/cgi/", "sample.cgi", 'pass=a amrita2_user=on comment=aaa')
      assert_equal('Amrita2 CGI and HTML generator sample', r.elements['body/h1'].text)
      f = r.elements['body/form']
      assert_equal('post', f.attributes['method'])
      input = f.elements.to_a('table/tr/td/input')
      assert_equal('', input[0].attributes['value'])
      assert_equal('name', input[0].attributes['name'])
      assert_equal('a', input[1].attributes['value'])
      assert_equal('pass', input[1].attributes['name'])
      assert_equal('on', input[2].attributes['checked'])
      assert_equal('amrita2_user', input[2].attributes['name'])
      assert_equal('checkbox', input[2].attributes['type'])

      assert_equal('comment', f.elements['table/tr/td/textarea'].attributes['name'])
      assert_equal('aaa', f.elements['table/tr/td/textarea'].text)
      assert_equal('amrita2_user=on/comment=aaa/pass=a/', r.elements['body/span'].text)
    end
  end

  class TestDialog < Test::Unit::TestCase
    PORT=12345
    def setup
      require 'net/http'
      $:  << "#{PRJ_DIR}/sample/dialog/"
      require "#{PRJ_DIR}/sample/dialog/hello.rb"
      http_opt = { 
        :Port => PORT, 
        :Logger=>WEBrick::Log.new(nil, WEBrick::Log::FATAL),
        :AccessLog=>[]
      }
      @httpd = WEBrick::HTTPServer.new(http_opt)
      @httpd.mount('/hello', DialogServlet, HelloDialog)
      Thread.start do
        @httpd.start
      end
    end

    def teardown
      @httpd.shutdown
    end

    def do_get(path)
      Net::HTTP.start('localhost', PORT) do |http|
        response = http.get(path)
        res = response.body
        return REXML::Document.new(res).root
      end
    end

    def do_post(path, query='')
      Net::HTTP.start('localhost', PORT) do |http|
        req = Net::HTTP::Post.new(path)
        req['Cookie'] = "_session_id=d#{@session_id}" if  @session_id
        response = http.request(req, query)
        response.each_header do |k,v|
          #p k,v
        end
        @session_id = $1 if response['set-cookie'] =~/_session_id=(\w+)/
        res = response.body
        return REXML::Document.new(res).root
      end
    end

    def test_hello
      r = do_get('/hello')
      tr = r.elements.to_a('body/form/table/tr')
      assert_equal("name", tr[0].elements['th'].text)
    end

    def test_hello2
      r = do_get('/hello')
      tr = r.elements.to_a('body/form/table/tr')
      assert_equal("name", tr[0].elements['th'].text)

      r = do_post('/hello', 'name=aaa&ok=ok')
      #p r.to_s
      tr = r.elements.to_a('body/form/table/tr')
      #p tr[0].elements['td/span'].to_s
      assert_equal("Hello, aaa !", tr[0].elements['td/span'].text)

      r = do_post('/hello', 'ok=ok')
      r = do_post('/hello', 'name=zzz&ok=ok')
      tr = r.elements.to_a('body/form/table/tr')
      assert_equal("Hello, zzz !", tr[0].elements['td/span'].text)
    end

  end
end

if __FILE__ == $0
  include Amrita2SampleTest
  cname = ARGV.shift
  if cname
    method = ARGV.shift
    if method
      eval "Test::Unit::UI::Console::TestRunner.run(#{cname}.new(\"#{method}\"))"
    else
      cls = eval(cname)
      Test::Unit::UI::Console::TestRunner.run(cls)
    end
  else
    suite = Test::Unit::TestSuite.new
    suite << TestDialog.suite
    suite << TestCgi.suite
    suite << TestSpec.suite
    suite << TestHello.suite
    Test::Unit::UI::Console::TestRunner.run(suite)
  end
end
